# INPUTS by (Prayz Jomba)
# last updated(11 Oct 2021) 02:03:05 PM

import argparse


#CHECKING INPUTS
par = argparse.ArgumentParser(prog='yw', formatter_class=argparse.RawTextHelpFormatter, description='Watch Youtube videos with mpv while saving to disk.')
par.add_argument('Quality', nargs='?', help=
'''
1/8 = 1080p | 2 = 240p  | 3 = 360p  | 4 = 480p
7 = 720p    | 9 = 1440p | 0 = 2160p    
w = worst   | b = best  | a = audio(aac)
o = audio(opus)  | O = audio(Best Opus)
default is 480p''')

par.add_argument('-b', '--disable-bar', action='store_true', help='disable bar')
par.add_argument('-c', '--disable-clear', action='store_true', help="don't clear the screen")
par.add_argument('-d', '--dir', action='store_true', help="save to current directory")
par.add_argument('-f', '--fullscreen', action='store_true', help="enable fullscreen")
par.add_argument('-p', '--playonly', action='store_true', help="play only, don't save")
par.add_argument('-m', '--mkv', action='store_true', help="play mkv, default is mp4")
par.add_argument('-r', '--disable-resume', action='store_true', help="don't resume, start at the beginning\n(wont work on live videos)")
par.add_argument('-s', '--size', action='store_true', help="show size and exit")
par.add_argument('-sk', '--size-mkv', action='store_true', help="show mkv size, don't forget the '-m' option")
par.add_argument('-sp', '--size-mp4', action='store_true', help="show mp4 size")
par.add_argument('-w', '--watch', action='store_true', help="play after streaming")

args = par.parse_args()



# MENU by (Prayz Jomba)
# last updated(10 Sep 2021) 11:13:18 AM


from rich.panel import Panel
from rich.padding import Padding
from rich import print as rprint


# COLORS
r   = 'color(9)'        #red
lg  = 'color(112)'      #light green
tc  = 'color(155)'      #lightest green 
w   = 'color(254)'      #white
g   = 'b color(46)'     #green
b   = 'color(80)'       #blue
aq   = 'color(78)'      #aqua
y   = 'b color(190)'    #yellow
pu  = 'color(169)'      #purple
br  = 'b color(172)'    #brown
yo  = 'b color(220)'    #yellow/orange
ly  = 'b color(215)'    #light yellow


def con_err():
    ce = Panel.fit(f"[{yo}]Connection Error!![/{yo}]", border_style = yo)
    rprint(Padding(ce, (0, 28)))

def age_err():
    ce = Panel.fit(f"[{yo}]You have to sign in to watch this video!![/{yo}]", border_style = yo)
    rprint(Padding(ce, (0, 16)))

def tim_err(link):
    ce = Panel.fit(f"[{yo}]Check Your LINK[/{yo}]", border_style = yo)
    lk = f'[b {w}]{link} [/b {w}]'
    rprint(Padding(ce, (0, 30)))
    rprint(Padding(lk, (0, 18)))

def usr_err():
    ce = Panel.fit(f"[{yo}]You Killed Me [/{yo}]", border_style = yo)
    rprint(Padding(ce, (0, 30)))

def title(link, title):
    title   = f"[b {w}]TITLE:[/b {w}] [{tc}]{title}[/{tc}]"
    rprint(f'{title} \n{link}')

def version(apn, ver):
    ap_name = f'[{br}]{apn}[/{br}]'
    ap_ver  = f'[{g}]{ver}[/{g}]'
    dev     = f'[{y}]Prayz Jomba[/{y}]'
    version = Panel.fit(f'[{w}]{ap_name} | ver({ap_ver}) | ({dev})', border_style = w)
    rprint(Padding(version, (0,20)))

def title(link, title):
    link    = f'[b {w}]LINK: {link} [/b {w}]'
    title   = f"[b {w}]TITLE:[/b {w}] [{tc}]{title}[/{tc}]"
    rprint(f'{title} \n{link}')

def bestQ(quality):
    global bestq

def duration_asize(duration, aac, opus):

    def s(size):
        if size is None:
            return f'[{r}]?(MB)[/{r}]'
        else:
            audioSize = size.split()
            sizeN = f'[{lg}]{audioSize[0]}[/{lg}]'
            sizeT = audioSize[1]
            if 'G' in sizeT:
                sizeT = f'[{pu}]{sizeT}[/{pu}]'
            else:
                sizeT = f'[{ly}]{sizeT}[/{ly}]'
            return sizeN+sizeT
    s_aac = s(aac)
    s_opus = s(opus)
    duration = f'[b {w}]DURATION:[/b {w}][{ly}] ([i {lg}]{duration}[/i {lg}])[/{ly}]'
    A = f'[b {b}]AAC:[/b {b}] [b {lg}]{s_aac}[/b {lg}]'
    O = f'[b {aq}]OPUS:[/b {aq}] [b {lg}]{s_opus}[/b {lg}]'

    #print(len(duration))
    #print(len(A))

    #dur_len
    if   len(duration) == 103: duration += '        '
    elif len(duration) == 104: duration += '       '
    elif len(duration) == 106: duration += '     '
    elif len(duration) == 107: duration += '    '
    elif len(duration) == 108: duration += '   '

    #aac_len
    if   len(A) == 87: A += '         '
    elif len(A) == 116: A += '         '
    elif len(A) == 117: A += '        '
    elif len(A) == 118: A += '       '
    elif len(A) == 119: A += '      '
    elif len(A) == 122: A += '       '
    elif len(A) == 123: A += '      '
    elif len(A) == 124: A += '     '
    elif len(A) == 125: A += '    '
    #print(len(A), len(O), len(duration))
    rprint(f'{duration}| {A}| {O}\n')


def clr_size(name, size):
    if type(size) is tuple:
        size = size[0]
    if type(size) is list:
        bestq = size[1]
        size  = size[0]

    if size is None:
        return ''

    elif len(size.split()) != 2:
        size = f'[{r}]{size}(MB)[/{r}]'
    else:
        size = size.split()
        ss = size[0]
        st = size[1]
        ss = f'[b {lg}]{ss}[/b {lg}]'
        if 'G' in st:
            st = f'[{pu}]{st}[/{pu}]'
        else:
            st = f'[{ly}]{st}[/{ly}]'
        size = ss+st
    if name == 'best':
        q = f' [{br}]({bestq})[/{br}]'
        size = size+q
    name = f'[{br}]{name}[/{br}]'

    sizeQu = f'{name} = {size}'
    #print(len(sizeQu))
    sizeQu = lens(sizeQu)
    sizeQu = [sizeQu]
    return sizeQu


def lens(size):
    if   len(size) == 62: size += '       '
    elif len(size) == 63: size += '      '
    elif len(size) == 97: size += '     '
    elif len(size) == 98: size += '    '
    elif len(size) == 99: size += '   '
    elif len(size) == 100: size += '  '
    elif len(size) == 101: size += '     '
    elif len(size) == 102: size += '    '
    elif len(size) == 103: size += '   '
    elif len(size) == 104: size += '  '
    elif len(size) == 106: size += '       '
    elif len(size) == 109: size += '         '
    elif len(size) == 107: size += '    '
    elif len(size) == 108: size += '   '
    elif len(size) == 122: size += '       '
    elif len(size) == 124: size += '     '
    elif len(size) == 125: size += '    '
    elif len(size) == 128: size += '   '
    return size


def data(fmat, qsize):
    qlen = len(qsize)
    if qlen == 1:
        rprint(f'[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}[/{ly}]\n')
    elif qlen == 2:
        rprint(f'[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}[/{ly}]\n')
    elif qlen == 3:
        rprint(f'[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}| {qsize[2]}[/{ly}]\n')
    elif qlen == 4:
        rprint(f'''[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}| {qsize[2]}
     {qsize[3]}[/{ly}]\n''')
    elif qlen == 5:
        rprint(f'''[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}| {qsize[2]}
     {qsize[3]}| {qsize[4]}[/{ly}]\n''')
    elif qlen == 6:
        rprint(f'''[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}| {qsize[2]}
     {qsize[3]}| {qsize[4]}| {qsize[5]}[/{ly}]\n''')
    elif qlen == 7:
        rprint(f'''[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}| {qsize[2]}
     {qsize[3]}| {qsize[4]}| {qsize[5]}
     {qsize[6]}[/{ly}]\n''')
    elif qlen == 8:
        rprint(f'''[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}| {qsize[2]}
     {qsize[3]}| {qsize[4]}| {qsize[5]}
     {qsize[6]}| {qsize[7]}[/{ly}]\n''')
    elif qlen == 9:
        rprint(f'''[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}| {qsize[2]}
     {qsize[3]}| {qsize[4]}| {qsize[5]}
     {qsize[6]}| {qsize[7]}| {qsize[8]}[/{ly}]\n''')
    elif qlen == 10:
        rprint(f'''[b {w}]{fmat}:[/b {w}][{ly}] {qsize[0]}| {qsize[1]}| {qsize[2]}
     {qsize[3]}| {qsize[4]}| {qsize[5]}
     {qsize[6]}| {qsize[7]}| {qsize[8]}
     {qsize[9]}[/{ly}]\n''')


def live():
    live = Panel.fit(f'[{w}]This Video is [{br}]LIVE[/{br}]',border_style = br)
    rprint(Padding(live, (0,27)))

def premier(duration):
    premier = Panel.fit(f'[{br}]PREMIERING NOW[/{br}]', border_style = br)
    duration = f'[b {w}]DURATION:[/b {w}][{ly}] ([i {lg}]{duration}[/i {lg}])[/{ly}]'
    rprint(f'{duration}')
    rprint(Padding(premier, (0, 30)))

def liveFormats(formats):
    fList = [i for i in formats if i is not None]
    LL = len(fList)

    if 92 in fList:
        fList[fList.index(92)] = f'[{br}]240p[/{br}]'
    if 93 in fList:
        fList[fList.index(93)] = f'[{br}]360p[/{br}]'
    if 94 in fList:
        fList[fList.index(94)] = f'[{br}]480p[/{br}]'
    if 95 in fList:
        fList[fList.index(95)] = f'[{br}]720p[/{br}]'
    if 96 in fList:
        fList[fList.index(96)] = f'[{br}]1080p[/{br}]'

    lv = f'[b {w}]LIVE:[/b {w}][{y}]'
    if LL == 1:
        rprint(f'{lv} {fList[0]}[/{y}]')
    elif LL == 2:
        rprint(f'{lv} {fList[0]} | {fList[1]}[/{y}]')
    elif LL == 3:
        rprint(f'{lv} {fList[0]} | {fList[1]} | {fList[2]}[/{y}]')
    elif LL == 4:
        rprint(f'{lv} {fList[0]} | {fList[1]} | {fList[2]} | {fList[3]}[/{y}]')
    elif LL == 5:
        rprint(f'{lv} {fList[0]} | {fList[1]} | {fList[2]} | {fList[3]} | {fList[4]}[/{y}]')


def checker(listt, size, item, name):
    if size:
        result = bool([i for i in listt if i.endswith(item)])
        if result:
            return name
        else:
            if bool([i for i in listt if i.endswith('480')]):
                return f'[s {r}]{name}[/s {r}] 480p'
            elif bool([i for i in listt if i.endswith('360')]):
                return f'[s {r}]{name}[/s {r}] 360p'
            else:
                return f'[s {r}]{name}[/s {r}] best'
    else:
        return name


def play(choice, size, mkv, video_quality):
    aac = video_quality.get('aac')
    opus = video_quality.get('opus')

    def bq():
        if mkv:
            if bool([i for i in video_quality if i.endswith('mkv_480')]):
                return '244+249/244+250/244+251/333+249/333+250/333+251'
            elif bool([i for i in video_quality if i.endswith('mkv_360')]):
                return '243+249/243+250/243+251'
            else:
                return '94/93/bestvideo[ext=webm]+bestaudio[ext=webm]/best'
        else:
            if bool([i for i in video_quality if i.endswith('mp4_480')]):
                return '135+140/397+140'
            elif bool([i for i in video_quality if i.endswith('mp4_360')]):
                return '134+140/396+140'
            else:
                return '94/93/best'
            

    if choice == 'a':
        if aac:
            qInfo = 'Audio'
        else:
            qInfo = f'[s {r}]AAC[/s {r}] 480p (DEFAULT)'
        qPlay = aac
    elif choice == 'o':
        if opus:
            qInfo = 'Opus'
        else:
            qInfo = f'[s {r}]OPUS[/s {r}] 480p (DEFAULT)'
        qPlay = opus

    elif choice == 'O':
        if opus:
            qInfo = 'Best Opus'
        else:
            qInfo = f'[s {r}]OPUS[/s {r}] 480p (DEFAULT)'
        qPlay = 'bestaudio[ext=webm]'

    elif choice == 'b':
        qInfo = 'Best'
        if mkv:
            qPlay = 'bestvideo[ext=webm]+bestaudio[ext=webm]'
        else:
            qPlay = 'best'

    elif choice == 'w': 
        qInfo = f'[{r}]Worst[/{r}]'
        if mkv:
            qPlay = 'worstvideo[ext=webm]+worstaudio[ext=webm]'
        else:
            qPlay = 'worst'

    elif choice == '1' or choice == '8':
        if mkv:
            qInfo = checker(video_quality, size, 'mkv_1080', '1080p')
            vid = video_quality.get('mkv_1080')
            qPlay = f'96/{vid}+{opus}/{bq()}'
        else:
            qInfo = checker(video_quality, size, 'mp4_1080', '1080p')
            vid = video_quality.get('mp4_1080')
            qPlay = f'96/{vid}+{aac}/{bq()}'

    elif choice == '2': 
        if mkv:
            qInfo = checker(video_quality, size, 'mkv_240', '240p')
            vid = video_quality.get('mkv_240')
            qPlay = f'92/{vid}+{opus}/{bq()}'
        else:
            qInfo = checker(video_quality, size, 'mp4_240', '240p')
            vid = video_quality.get('mp4_240')
            qPlay = f'92/{vid}+{aac}/{bq()}'

    elif choice == '3': 
        if mkv:
            qInfo = checker(video_quality, size, 'mkv_360', '360p')
            vid = video_quality.get('mkv_360')
            qPlay = f'93/{vid}+{opus}/{bq()}'
        else:
            qInfo = checker(video_quality, size, 'mp4_360', '360p')
            vid = video_quality.get('mp4_360')
            qPlay = f'93/{vid}+{aac}/{bq()}'

    elif choice == '4': 
        qInfo = '480p'
        if mkv:
            qInfo = checker(video_quality, size, 'mkv_480', '480p')
            vid = video_quality.get('mkv_480')
            qPlay = f'94/{vid}+{opus}/{bq()}'
        else:
            qInfo = checker(video_quality, size, 'mp4_480', '480p')
            vid = video_quality.get('mp4_480')
            qPlay = f'94/{vid}+{aac}/{bq()}'

    elif choice == '7':
        if mkv:
            qInfo = checker(video_quality, size, 'mkv_720', '720p')
            vid = video_quality.get('mkv_720')
            qPlay = f'95/{vid}+{opus}/{bq()}'
        else:
            qInfo = checker(video_quality, size, 'mp4_720', '720p')
            vid = video_quality.get('mp4_720')
            qPlay = f'95/{vid}+{aac}/{bq()}'

    elif choice == '9':
        if mkv:
            qInfo = checker(video_quality, size, 'mkv_1440', '1440p')
            vid = video_quality.get('mkv_1440')
            qPlay = f'{vid}+{opus}/{bq()}'
        else:
            qInfo = checker(video_quality, size, 'mp4_1440', '1440p')
            vid = video_quality.get('mp4_1440')
            qPlay = f'{vid}+{aac}/{bq()}'

    elif choice == '0':
        if mkv:
            qInfo = checker(video_quality, size, 'mkv_2160', '2160p')
            vid = video_quality.get('mkv_2160')
            qPlay = f'{vid}+{opus}/{bq()}'
        else:
            qInfo = checker(video_quality, size, 'mp4_2160', '2160p')
            vid = video_quality.get('mp4_2160')
            qPlay = f'{vid}+{aac}/{bq()}'

    else: 
        qPlay = f'{bq()}'

    if choice:
        rprint(f'[{w}]PLAYING:[/{w}] [{lg}]{qInfo}[/{lg}]')
    else:
        if bool([i for i in video_quality if i.endswith('480')]):
            rprint(f'[{w}]PLAYING:[/{w}] [{lg}]480p (DEFAULT)[/{lg}]')
        elif bool([i for i in video_quality if i.endswith('360')]):
            rprint(f'[{w}]PLAYING:[/{w}] [{lg}]360p[/{lg}]')
        else:
            rprint(f'[{w}]PLAYING:[/{w}] [{lg}]best[/{lg}]')

    if size:
        if choice == 'o' or choice == 'O':
            rprint(f'[{w}]OUTPUT:[/{w}] [{b}]opus[/{b}]')
        elif choice == 'a':
            rprint(f'[{w}]OUTPUT:[/{w}] [{b}]aac[/{b}]')
        elif mkv:
            rprint(f'[{w}]OUTPUT:[/{w}] [{b}]mkv[/{b}]')
        else:
            rprint(f'[{w}]OUTPUT:[/{w}] [{yo}]mp4[/{yo}]')
    else:
        if mkv:
            rprint(f'[{w}]OUTPUT:[/{w}][s {r}]mkv[/s {r}] [{yo}]mp4[/{yo}]')
        else:
            rprint(f'[{w}]OUTPUT:[/{w}] [{yo}]mp4[/{yo}]')

    return qPlay











#!/usr/bin/python

# (yw) YOUTUBE-WATCH (Prayz Jomba)
#           >>>VERSION (0.6)<<<
#

# APP
ver = '0.6'
apn = 'yWatch'


# IMPORTS
import os, subprocess
from ywatch import userInputs, dataprocess, menu
from datetime import date


# USER INPUTS
userIn  = userInputs.args
user_q  = userIn.Quality


# DATA FILE
dataFile = dataprocess.dataFile
link = dataprocess.link

# CLEAR SCREEN
if not userIn.disable_clear: #-c
    os.system('clear')

# FULL SCREEN
if userIn.fullscreen: #-f
    fs = '-fs'
else:
    fs = '--no-fs'


# FUNCTIONS
def menu_version():
    if userIn.disable_bar:   #-b
        print('')
    else:
        menu.version(apn, ver)

def menu_title():
    menu.title(link, title)

def dataFileSize():
    return os.path.getsize(dataFile)

def savedLink():
    return dataprocess.Read().link()

def save_title():
    dataprocess.Save().linkAndTitle()

def save_data():
    dataprocess.Save().data()

def read_title():
    global duration, title
    title    = dataprocess.Read().title()
    duration = dataprocess.Read().duration()

def size_true():
    return dataprocess.Read().true()

def menu_data():
    dataprocess.Read().duration_menu()
    if userIn.size_mkv:
        dataprocess.Read().size_mkv()
    elif userIn.size_mp4:
        dataprocess.Read().size_mp4()
    else:
        dataprocess.Read().size_mkv()
        dataprocess.Read().size_mp4()

def video_quality():
    vq = (dataprocess.qualito)
    vq = {k:v for k,v in list(vq.items()) if v is not None}
    for k,v in vq.items():
        if type(v) == tuple:
            vq[k] = v[1]
    return vq

def live_formats():
    dataprocess.Read().live()

def stream():
    dataprocess.stream(link, quality, file, resume, userIn.playonly, fs)


# CONDITIONS
if not os.path.isfile(dataFile):
    save_title()

if link == savedLink() and dataFileSize() > 60:
    read_title()
    menu_version()
    menu_title()

else:
    save_title()
    read_title()
    menu_version()
    menu_title()

if duration == '0':
    menu.live()
    if userIn.size:
        exit()

if dataFileSize() < 400:
    save_data()

if duration != '0' and not size_true():
    menu.premier(duration)

if size_true():
    menu_data()
else:
    live_formats()


if userIn.size:
    exit()


size_true = size_true()
quality = menu.play(userIn.Quality, size_true, userIn.mkv, video_quality())


# DATE
date  = date.today()
today = date.strftime(f"Videos of ({date}) %a")

# CHANGE DIR
home = os.path.expanduser('~')  
if userIn.dir:
    folder = f'WATCHED/{today}'
else:
    folder = f'{home}/Videos/WATCHED/{today}'

if not os.path.isdir(folder):
    os.makedirs(folder)  
os.chdir(folder)   

x = 1

#AUDIO DIR
if user_q == 'a' or user_q == 'o' or user_q == 'O' and size_true:
    if not os.path.isdir('Audio'):
        os.makedirs('Audio') 
    os.chdir('Audio')

    if user_q == 'a':
        file = f'{title}.aac'
        if os.path.isfile(file):
            while os.path.isfile(f'{title}{x}.aac'):
                x += 1
            file = f'{title}{x}.aac'
    elif user_q == 'o':
        file = f'{title}.opus'
        if os.path.isfile(file):
            while os.path.isfile(f'{title}{x}.opus'):
                x += 1
            file = f'{title}{x}.opus'
    else:
        file = f'{title} (Best Opus).opus'
        if os.path.isfile(file):
            while os.path.isfile(f'{title}{x} (Best Opus).opus'):
                x += 1
            file = f'{title}{x} (Best Opus).opus'

elif userIn.mkv:
    file = f'{title}.mkv'
    if os.path.isfile(file):
        while os.path.isfile(f'{title}{x}.mkv'):
            x += 1
        file = f'{title}{x}.mkv'

else:
    file = f'{title}.mp4'
    if os.path.isfile(file):
        while os.path.isfile(f'{title}{x}.mp4'):
            x += 1
        file = f'{title}{x}.mp4'


#LIVE DIR
if duration == '0' or not size_true:
    if not os.path.isdir('Live'):
        os.makedirs('Live')
    os.chdir('Live')

    file = f'{title}.ts'
    if os.path.isfile(file):
        while os.path.isfile(f'{title}_{x}.ts'):
            x += 1
        file = f'{title}_{x}.ts'
    

#RESUME
if userIn.disable_resume:
    resume = '--no-resume-playback'
else:
    resume = '--resume-playback'


stream()


#TS TO MP4
if file.endswith('ts'):
    file_mp4 = f'{file[:-2]}mp4'
    if os.path.isfile(file_mp4):
        file_mp4 = file_mp4[:-4]
        while os.path.isfile(f'{file_mp4}_{x}.mp4'):
            x += 1
        file_mp4 = f'{file_mp4}_{x}.mp4'
    os.rename(file, file_mp4)
    file = file_mp4


#OPEN
if userIn.watch:
    subprocess.Popen(['mpv', fs, file, '--loop-playlist', '--player-operation-mode=pseudo-gui'])



#END



#   CHECK LINK by (Prayz Jomba)
#   last updated(01 Sep 2021) 12:49:20 PM


import clipboard
from rich.panel import Panel
from rich.padding import Padding
from rich import print as rprint



# COLORS
clink       = '[b color(112)]LINK[/b color(112)]'
notfound    = '[b color(254)]not found[/b color(254)]'
invalid     = '[b color(220)]Invalid[/b color(220)]'
bdr         = 'color(9)'


# LINK CHECK
link = clipboard.paste()
link = ''.join(link.split() [:1])


if not link:
    empty = Panel.fit(f"{clink} {notfound}", border_style = bdr)
    rprint(Padding(empty, (0, 30)))
    exit()

elif not link.startswith('https://') or 'you' not in link:
    invalid = Panel.fit(f"{invalid} {clink}", border_style = bdr)
    rprint(Padding(invalid, (0, 31)))
    exit()



# DATAPROCESS by (Prayz Jomba)
# last updated(10 sep 2021) 11:03:19 AM


import subprocess, re, os, tempfile
from time import sleep
from ywatch.checklink import link
import ywatch.menu as menu


# CHANGE DIR
tmp_dir = tempfile.gettempdir()

dataFile = f'{tmp_dir}/wFile.txt'
qualito = dict()


class Save:

    def linkAndTitle(self):
        with open(dataFile, 'w') as save:
            save.write(link) 

            try:
                dt = subprocess.run(['youtube-dl', '-e', '--get-duration', link, '--no-playlist'], timeout=15, capture_output=True)
                dtOut = dt.stdout.decode('utf-8')
                dtErr = dt.stderr.decode('utf-8')
                if dtOut:
                    save.write(f'\n{dtOut}')
                else:
                    if 'confirm your age' in dtErr:
                        menu.age_err()
                    elif 'Unable to download' in dtErr:
                        menu.con_err()
                    else:
                        menu.tim_err(link)
                    exit()
            except subprocess.TimeoutExpired:
                print(''); menu.tim_err(link)
                exit()
            except KeyboardInterrupt:
                print(''); menu.usr_err()
                exit()

    def data(self):
        with open(dataFile, 'a') as save_data:
            try:
                dd =  subprocess.run(['youtube-dl', '-F', link, '--no-playlist'], timeout=15, capture_output=True)
                ddOut = dd.stdout.decode('utf-8')
                if ddOut:
                    save_data.write(ddOut)
                else:
                    menu.con_err()
                    exit()
            except subprocess.TimeoutExpired:
                print(''); menu.tim_err(link)
                exit()
            except KeyboardInterrupt:
                print(''); menu.usr_err()
                exit()


class Read:
    def __init__(self):
        with open(dataFile, 'r') as data:
            self.data = data.read()
            self.dataL = self.data.splitlines()

    def link(self):
        return self.dataL[0]
    
    def title(self):
        return re.sub('[%\/|`$^]', '_', self.dataL[1])

    def duration(self):
        duration = self.dataL[2]
        if len(duration) == 2:
            duration = f'0:{duration}'
        return duration
    

    def listcheck(self, *quality):
        quality = list(quality)

        while len(quality) > 1 and not Read.qcheck(self, quality[0]):
            quality.pop(0)

        if bool(Read.qcheck(self, quality[0])):
            qlt = quality[0].split('\\')[0]
            return Read.qcheck(self, quality[0]), qlt

    def qcheck(self, quality):
        q = re.search(quality, self.data)
        if q:
            if q[0].startswith('9'):
                q = int(q[0].split() [0])
            else:
                q = Read.size_check(self, q[0])
        return q

    def size_check(self, *file):
        file_list = file
        file_one = len(file_list) == 1
        file = file_list[0]

        mkv = re.search('.*webm', file)
        if mkv:
            mkvq = mkv[0].split()[0]

        fileS = re.search('.*iB', file)
        if fileS is None:
            fileS = '?'
            return fileS
        else:
            fileS = fileS[0].split() [-1]
            if fileS[-3:] == 'KiB':
                fileS = float(fileS[:-3])
                fileS = round(fileS * 0.001024, 1)
                if mkv:
                    if mkvq not in ['249', '250', '251']:
                        opus = float(Read._opus(self)[:-5])
                        opusBest = float(Read._opusBest(self)[:-5])
                        if file_one:
                            fileS = round(fileS + opus, 1)
                        else:
                            fileS = round(fileS + opusBest, 1)

                elif not file.startswith('140') and not file.endswith('best'):
                    aac = float(Read._audio(self)[:-5])
                    fileS = round(fileS + aac, 1)
                fileS = f'{fileS} (MB)'
                return fileS

            elif fileS[-3:] == 'MiB':
                fileS = float(fileS[:-3])
                fileS = round(fileS * 1.048576, 1)
                if mkv:
                    if mkvq not in ['249', '250', '251']:
                        if file_one:
                            if not 'G' in Read._opus(self):
                                opus = float(Read._opus(self)[:-5])
                            else:
                                opus = float(Read._opus(self)[:-5]) * 1073.742
                            fileS = round(fileS + opus, 1)

                        else:
                            if not 'G' in Read._opusBest(self):
                                opusBest = float(Read._opusBest(self)[:-5])
                            else:
                                opusBest = float(Read._opusBest(self)[:-5]) * 1073.742
                            fileS = round(fileS + opusBest, 1)

                elif not file.startswith('140') and not file.endswith('best'):
                    if not 'G' in Read._audio(self):
                        aac = float(Read._audio(self)[:-5])
                    else:
                        aac = float(Read._audio(self)[:-5]) * 1073.742
                    fileS = round(fileS + aac, 1)

                if fileS > 1000:
                    fileS = round(fileS * 0.001, 2)
                    fileS = f'{fileS} (GB)'
                else:
                    fileS = f'{fileS} (MB)'
                return fileS

            else:
                fileS = float(fileS[:-3])
                fileS = round(fileS * 1.073742, 2)
                if mkv:
                    if mkvq not in ['249', '250', '251']:
                        if file_one:
                            if not 'G' in Read._opus(self):
                                opus = float(Read._opus(self)[:-5]) * 0.001
                            else:
                                opus = float(Read._opus(self)[:-5]) * 1.073742
                            fileS = round(fileS + opus, 2)

                        else:
                            if not 'G' in Read._opusBest(self):
                                opusBest = float(Read._opusBest(self)[:-5]) * 0.001
                            else:
                                opusBest = float(Read._opusBest(self)[:-5]) * 1.073742
                            fileS = round(fileS + opusBest, 2)

                elif not file.startswith('140') and not file.endswith('best'):
                    if not 'G' in Read._audio(self):
                        aac = float(Read._audio(self)[:-5]) * 0.001
                        fileS = round(fileS + aac, 2)
                    else:
                        aac = float(Read._audio(self)[:-5]) * 1.073742
                        fileS = round(fileS + aac, 2)
                fileS = f'{fileS} (GB)'
            return fileS

    def true(self):
        if (bool(re.search('KiB', self.data))):
            check = re.search('KiB', self.data)
        elif (bool(re.search('MiB', self.data))):
            check = re.search('MiB', self.data)
        else:
            check = re.search('GiB', self.data)
        return check


    def _audio(self):
        aac_size = Read.listcheck(self, '140\s.*m4a\s.*audio only.*')
        qualito['aac'] = aac_size[1]
        return aac_size[0]
    
    def _opus(self):
        sOpus = Read.listcheck(self, '249\s.*webm\s.*audio only.*', '250\s.*webm\s.*audio only.*', '251\s.*webm\s.*audio only.*')
        qualito['opus'] = sOpus[1]
        return sOpus[0]
    
    def _opusBest(self):
        bOpus = re.findall('.*webm\s.*audio only.*', self.data) [-1]
        size = Read.size_check(self, bOpus, 'A')
        return size
    
    def _best(self):
        bestS = re.search('.*best', self.data) [0]
        bestq = bestS.split() [3]
        bestS = Read.size_check(self, bestS)
        return [bestS, bestq]

    def duration_menu(self):
        aac = Read._audio(self)
        opus = Read._opus(self)
        menu.duration_asize(Read.duration(self), aac, opus)

    def size_mp4(self):
        best    = Read._best(self)
        sAAC    = Read._audio(self)
        p240    = Read.listcheck(self, '133\s.*mp4\s.*video only.*', '395\s.*mp4\s.*video only.*')
        p360    = Read.listcheck(self, '134\s.*mp4\s.*video only.*', '396\s.*mp4\s.*video only.*')
        p480    = Read.listcheck(self, '135\s.*mp4\s.*video only.*', '397\s.*mp4\s.*video only.*')
        p720    = Read.listcheck(self, '136\s.*mp4\s.*video only.*', '298\s.*mp4\s.*video only.*', '398\s.*mp4\s.*video only.*', '698\s.*mp4\s.*video only.*')
        p1080   = Read.listcheck(self, '137\s.*mp4\s.*video only.*', '399\s.*mp4\s.*video only.*', '299\s.*mp4\s.*video only.*', '699\s.*mp4\s.*video only.*')
        p1440   = Read.listcheck(self, '400\s.*mp4\s.*video only.*', '700\s.*mp4\s.*video only.*')
        p2160   = Read.listcheck(self, '401\s.*mp4\s.*video only.*', '701\s.*mp4\s.*video only.*')


        qqq = []
        #qqq += menu.clr_size('Audio', sAAC)
        qqq += menu.clr_size('240p', p240)
        qqq += menu.clr_size('360p', p360)
        qqq += menu.clr_size('480p', p480)
        qqq += menu.clr_size('720p', p720)
        qqq += menu.clr_size('1080p', p1080)
        qqq += menu.clr_size('1440p', p1440)
        qqq += menu.clr_size('2160p', p2160)
        qqq += menu.clr_size('best', best)

        menu.data('MP4', qqq)

        qualito['mp4_240'] = p240
        qualito['mp4_360'] = p360
        qualito['mp4_480'] = p480
        qualito['mp4_720'] = p720
        qualito['mp4_1080'] = p1080
        qualito['mp4_1440'] = p1440
        qualito['mp4_2160'] = p2160


    def mkvBest(self):
        bestV = re.findall('.*webm.*', self.data) [-1]
        size = Read.size_check(self, bestV, 'B')
        bestq = bestV.split() [3]
        return [size, bestq]


    def size_mkv(self):
        best    = Read.mkvBest(self)
        k240    = Read.listcheck(self, '242\s.*webm\s.*video only.*')
        k360    = Read.listcheck(self, '243\s.*webm\s.*video only.*')
        k480    = Read.listcheck(self, '244\s.*webm\s.*video only.*', '333\s.*webm\s.*video only.*')
        k720    = Read.listcheck(self, '247\s.*webm\s.*video only.*', '302\s.*webm\s.*video only.*', '334\s.*webm\s.*video only.*')
        k1080   = Read.listcheck(self, '248\s.*webm\s.*video only.*', '303\s.*webm\s.*video only.*', '335\s.*webm\s.*video only.*')
        k1440   = Read.listcheck(self, '271\s.*webm\s.*video only.*', '308\s.*webm\s.*video only.*', '336\s.*webm\s.*video only.*')
        k2160   = Read.listcheck(self, '313\s.*webm\s.*video only.*', '315\s.*webm\s.*video only.*', '337\s.*webm\s.*video only.*')

        qqq = []
        qqq += menu.clr_size('240p', k240)
        qqq += menu.clr_size('360p', k360)
        qqq += menu.clr_size('480p', k480)
        qqq += menu.clr_size('720p', k720)
        qqq += menu.clr_size('1080p', k1080)
        qqq += menu.clr_size('1440p', k1440)
        qqq += menu.clr_size('2160p', k2160)
        qqq += menu.clr_size('best', best)

        menu.data('MKV', qqq)

        qualito['mkv_240'] = k240
        qualito['mkv_360'] = k360
        qualito['mkv_480'] = k480
        qualito['mkv_720'] = k720
        qualito['mkv_1080'] = k1080
        qualito['mkv_1440'] = k1440
        qualito['mkv_2160'] = k2160


    def live(self):
        l240  = Read.qcheck(self, '92\s.*mp4')
        l360  = Read.qcheck(self, '93\s.*mp4')
        l480  = Read.qcheck(self, '94\s.*mp4')
        l720  = Read.qcheck(self, '95\s.*mp4')
        l1080 = Read.qcheck(self, '96\s.*mp4')
        formats = [l240, l360, l480, l720, l1080]
        menu.liveFormats(formats)


def stream(link, quality, file, resume, playonly, fs):
    try:
        if playonly:
            rec     = ''
            stream  = subprocess.run(['mpv', f'--ytdl-format={quality}/best', link, fs, resume, '--loop-playlist=no', rec, '--player-operation-mode=pseudo-gui'])
            if stream.returncode != 0:
                print(''); menu.con_err()
            exit()
        else:
            rec     = f'--stream-record={file}'
            stream  = subprocess.Popen(['mpv', f'--ytdl-format={quality}/best', link, fs, resume, '--loop-playlist=no', rec, '--player-operation-mode=pseudo-gui'])

            #WAITING..
            x = 0
            while not os.path.isfile(file):
                sleep(1)
                x += 1
                if x == 30:
                    print(''); menu.con_err()
                    stream.kill()
                    exit()

            #STREAMED FILE SIZE
            x = True
            while x:
                fsize = os.path.getsize(file)
                fsize = round(fsize * 0.000001, 1)
                print(f'\rSize = {fsize}(MB)', end='')
                if stream.poll() is not None:
                    print('\n')
                    x = False

    except KeyboardInterrupt:
        print(''); menu.usr_err()
        if not playonly: stream.kill()
        exit()






from setuptools import setup, find_packages


DESCRIPTION = "Watch Youtube videos with mpv while saving to disk."
LICENSE="GPLv3+"

setup(
    name="ywatch",
    version="0.6",
    author="Prayz Jomba",
    license=LICENSE,
    author_email="prayzjomba@protonmail.com",
    description=DESCRIPTION,
    long_description=DESCRIPTION,
    long_description_content_type="text/markdown",
    url="https://github.com/prayzjomba/yWatch",
    keywords=['mpv', 'youtube watch', 'ywatch', 'youtube-dl', 'youtube', 'mpv save', 'yw'],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
        "Intended Audience :: End Users/Desktop",
        "Environment :: Console",
        "Topic :: Multimedia :: Video",
        "Operating System :: POSIX :: Linux",
    ],

    packages=find_packages(),
    install_requires=['rich', 'clipboard'],
    entry_points={'console_scripts': ['yw=ywatch:yw']},
    python_requires=">=3.6",
)
